package main

import (
	"context"
	"database/sql"
	"errors"
	"fmt"
	"log"
	"strconv"
	"strings"

	"github.com/go-telegram/bot"
	"github.com/go-telegram/bot/models"
)

func defaultHandler(ctx context.Context, b *bot.Bot, update *models.Update) {
	// b.SendMessage(ctx, &bot.SendMessageParams{
	// 	ChatID: update.Message.Chat.ID,
	// 	Text:   "hello!",
	// })
	if update.Message != nil {
		log.Println(update.Message.Text)
	}
	log.Println(*update)
}

func registerChat(ctx context.Context, b *bot.Bot, update *models.Update) {
	msg := update.Message
	log.Println("registering", msg.Chat.ID)
	m, err := FetchMemberFromChat(ctx, b, msg.Chat.ID, msg.From.ID)
	if err == nil {
		if !IsAdmin(*m) {
			log.Println("register: user is not admin")
			return
		}
		args := strings.Split(msg.Text, " ")
		if len(args) == 1 {
			log.Println("register: there's no code")
			return
		}
		if !UseActivation(args[1]) {
			log.Println("register: wrong code: ", args[1])
			return
		}
		if err := ActivateChat(msg.Chat.ID); err != nil {
			log.Println("Error activating chat: ", err)
			return
		}
		b.DeleteMessage(ctx, &bot.DeleteMessageParams{ChatID: msg.Chat.ID, MessageID: msg.ID})
		sent, err := b.SendMessage(ctx, &bot.SendMessageParams{ChatID: msg.Chat.ID, Text: "Чат зарегистрирован", MessageThreadID: msg.MessageThreadID})
		if err == nil {
			err := DeleteAfterSeconds(msg.Chat.ID, sent.ID, 60)
			if err != nil {
				log.Println("register: failed to add to delete", err)
			}
		}
	} else {
		log.Println("register: error", err)
	}
}

func handleNewJoined(ctx context.Context, b *bot.Bot, u *models.Update) {
	for _, user := range u.Message.NewChatMembers {
		_, err := db.Exec(`INSERT INTO captchas (user_id, chat_id, message_id, correct_answer) values ($1, $2, 0, 0)`, user.ID, u.Message.Chat.ID)
		if err != nil {
			log.Println("newusers: can't add to db: ", err)
		}
		b.SendMessage(ctx, &bot.SendMessageParams{
			ChatID: u.Message.Chat.ID,
			Text:   "Check the capthca!",
			ReplyMarkup: models.InlineKeyboardMarkup{
				InlineKeyboard: [][]models.InlineKeyboardButton{
					{
						{URL: fmt.Sprintf("https://t.me/nefrace_php_test_bot?start=cap_%d", u.Message.Chat.ID), Text: "Check"},
					},
				},
			},
		})
	}
}

func banUser(ctx context.Context, b *bot.Bot, u *models.Update) {

}

func handlePrivateStartCaptcha(ctx context.Context, b *bot.Bot, u *models.Update) {
	args := strings.Split(u.Message.Text, " ")
	userID := u.Message.From.ID
	chatID := 0
	if len(args) > 1 {
		chat_str := strings.TrimPrefix(args[1], "cap_")
		id, err := strconv.Atoi(chat_str)
		if err == nil {
			chatID = id
		}
	}

	captcha := Captcha{}
	var err error
	if chatID != 0 {
		err = db.Get(&captcha, `select * from captchas where user_id = $1 and chat_id = $2`, userID, chatID)
		if errors.Is(err, sql.ErrNoRows) {
			b.SendMessage(ctx, &bot.SendMessageParams{
				Text:   " There's no captchas for that chat you came from.",
				ChatID: u.Message.Chat.ID,
			})
			return
		}
	} else {
		err = db.Get(&captcha, `select * from captchas where user_id = $1 and correct_answer != 0`, userID)
		if err != nil {
			b.SendMessage(ctx, &bot.SendMessageParams{
				Text:   " There's no captchas for that chat you came from.",
				ChatID: u.Message.Chat.ID,
			})
			return
		}
	}

	if captcha.CorrectAnswer == 0 {
		captcha.CorrectAnswer = 42
		msg, err := b.SendMessage(ctx, &bot.SendMessageParams{
			Text:   "Get me the answer!",
			ChatID: u.Message.Chat.ID,
		})
		if err == nil {
			captcha.MessageID = msg.ID
		}
		_, err = db.NamedExec("update captchas set correct_answer = :correct_answer, message_id = :message_id where id = :id", captcha)
		if err != nil {
			log.Println("Can't update captcha:", err)
		}
	} else {
		b.SendMessage(ctx, &bot.SendMessageParams{
			Text:   fmt.Sprintf("You already have captcha for chat %d", captcha.ChatID),
			ChatID: u.Message.Chat.ID,
		})
	}
}
